@@ -1,6 +1,6 @@
 use std::{collections::VecDeque, marker::PhantomData};
 
-use hecs::Entity;
+use hecs::{Column, Component, Entity};
 use hecs_schedule::GenericWorld;
 use smallvec::{smallvec, SmallVec};
 
@@ -10,17 +10,21 @@ const STACK_SIZE: usize = 64;
 
 /// Iterates children along with Query `Q`. Children who do not satisfy `Q` will be skipped.
 /// Count is known in advanced and will not fold iterator.
-pub struct ChildrenIter<'a, W, T> {
-    world: &'a W,
+pub struct ChildrenIter<'a, T: Component> {
+    children: Column<'a, Child<T>>,
     remaining: usize,
     current: Option<Entity>,
     marker: PhantomData<T>,
 }
 
-impl<'a, W: GenericWorld, T> ChildrenIter<'a, W, T> {
-    pub(crate) fn new(world: &'a W, num_children: usize, current: Option<Entity>) -> Self {
+impl<'a, T: Component> ChildrenIter<'a, T> {
+    pub(crate) fn new<W: GenericWorld>(
+        world: &'a W,
+        num_children: usize,
+        current: Option<Entity>,
+    ) -> Self {
         Self {
-            world,
+            children: world.try_get_column().unwrap(),
             remaining: num_children,
             current,
             marker: PhantomData,
@@ -28,9 +32,9 @@ impl<'a, W: GenericWorld, T> ChildrenIter<'a, W, T> {
     }
 }
 
-impl<'a, W: GenericWorld, T> Iterator for ChildrenIter<'a, W, T>
+impl<'a, T> Iterator for ChildrenIter<'a, T>
 where
-    T: 'static + Send + Sync,
+    T: Component,
 {
     type Item = Entity;
 
@@ -42,7 +46,7 @@ where
         self.remaining -= 1;
 
         let current = self.current?;
-        let data = match self.world.try_get::<Child<T>>(current) {
+        let data = match self.children.get(current) {
             Ok(data) => data,
             Err(_) => return None,
         };
@@ -63,33 +67,32 @@ where
     }
 }
 
-pub struct AncestorIter<'a, W, T> {
-    world: &'a W,
+pub struct AncestorIter<'a, T: Component> {
+    children: Column<'a, Child<T>>,
     current: Entity,
     marker: PhantomData<T>,
 }
 
-impl<'a, W: GenericWorld, T> AncestorIter<'a, W, T> {
-    pub(crate) fn new(world: &'a W, current: Entity) -> Self {
+impl<'a, T: Component> AncestorIter<'a, T> {
+    pub(crate) fn new<W: GenericWorld>(world: &'a W, current: Entity) -> Self {
         Self {
-            world,
+            children: world.try_get_column().unwrap(),
             current,
             marker: PhantomData,
         }
     }
 }
 
-impl<'a, W: GenericWorld, T: 'static + Send + Sync> Iterator for AncestorIter<'a, W, T> {
+impl<'a, T: Component> Iterator for AncestorIter<'a, T> {
     type Item = Entity;
 
     fn next(&mut self) -> Option<Self::Item> {
-        self.world
-            .try_get::<Child<T>>(self.current)
-            .ok()
-            .map(|child| {
-                self.current = child.parent;
-                child.parent
-            })
+        if let Some(child) = self.children.get(self.current).ok() {
+            self.current = child.parent;
+            Some(child.parent)
+        } else {
+            None
+        }
     }
 }
 
@@ -99,20 +102,25 @@ struct StackFrame {
     remaining: usize,
 }
 
-pub struct DepthFirstIterator<'a, W, T> {
-    world: &'a W,
+pub struct DepthFirstIterator<'a, T: Component> {
+    children: Column<'a, Child<T>>,
+    parents: Column<'a, Parent<T>>,
     marker: PhantomData<T>,
     /// Since StackFrame is so small, use smallvec optimizations
     stack: SmallVec<[StackFrame; STACK_SIZE]>,
 }
 
-impl<'a, W: GenericWorld, T: 'static + Send + Sync> DepthFirstIterator<'a, W, T> {
-    pub(crate) fn new(world: &'a W, root: Entity) -> Self {
-        let stack = world
-            .try_get::<Parent<T>>(root)
+impl<'a, T: Component> DepthFirstIterator<'a, T> {
+    pub(crate) fn new<W: GenericWorld>(world: &'a W, root: Entity) -> Self {
+        let children = world.try_get_column().unwrap();
+        let parents = world.try_get_column::<Parent<T>>().unwrap();
+
+        let stack = parents
+            .get(root)
+            .ok()
             .and_then(|parent| {
-                let first_child = parent.first_child(world)?;
-                Ok(smallvec![StackFrame {
+                let first_child = parent.first_child(world).ok()?;
+                Some(smallvec![StackFrame {
                     current: first_child,
                     remaining: parent.num_children,
                 }])
@@ -120,14 +128,99 @@ impl<'a, W: GenericWorld, T: 'static + Send + Sync> DepthFirstIterator<'a, W, T>
             .unwrap_or_default();
 
         Self {
-            world,
+            children,
+            parents,
             stack,
             marker: PhantomData,
         }
     }
 }
 
-impl<'a, W: GenericWorld, T: 'static + Send + Sync> Iterator for DepthFirstIterator<'a, W, T> {
+pub struct DepthFirstVisitor<'a, W, T: Component, F> {
+    world: &'a W,
+    children: Column<'a, Child<T>>,
+    parents: Column<'a, Parent<T>>,
+    marker: PhantomData<T>,
+    /// Since StackFrame is so small, use smallvec optimizations
+    stack: SmallVec<[StackFrame; STACK_SIZE]>,
+    accept: F,
+}
+
+impl<'a, F: Fn(&W, Entity) -> bool + Component, W: GenericWorld, T: Component>
+    DepthFirstVisitor<'a, W, T, F>
+{
+    pub(crate) fn new(world: &'a W, root: Entity, accept: F) -> Self {
+        let children = world.try_get_column().unwrap();
+        let parents = world.try_get_column::<Parent<T>>().unwrap();
+
+        let stack = parents
+            .get(root)
+            .ok()
+            .and_then(|parent| {
+                if (accept)(world, root) {
+                    let first_child = parent.first_child(world).ok()?;
+                    Some(smallvec![StackFrame {
+                        current: first_child,
+                        remaining: parent.num_children,
+                    }])
+                } else {
+                    None
+                }
+            })
+            .unwrap_or_default();
+
+        Self {
+            world,
+            accept,
+            children,
+            parents,
+            stack,
+            marker: PhantomData,
+        }
+    }
+}
+
+impl<'a, F: Fn(&W, Entity) -> bool + Component, W: GenericWorld, T: Component> Iterator
+    for DepthFirstVisitor<'a, W, T, F>
+{
+    type Item = Entity;
+
+    fn next(&mut self) -> Option<Self::Item> {
+        loop {
+            // The the topmost stackframe
+            let top = self.stack.last_mut()?;
+            // There are more children in current stackframe
+            if top.remaining > 0 {
+                let current = top.current;
+
+                let data = self.children.get(top.current).ok().unwrap();
+
+                // Go to the next child in the linked list of children
+                top.current = data.next;
+                top.remaining -= 1;
+
+                if !(self.accept)(self.world, current) {
+                    continue;
+                }
+
+                // If current is a parent, push a new stack frame with the first child
+                if let Ok(parent) = self.parents.get(current) {
+                    self.stack.push(StackFrame {
+                        current: parent.column_first_child(&self.children).unwrap(),
+                        remaining: parent.num_children,
+                    })
+                }
+
+                return Some(current);
+            } else {
+                // End of linked list of children, pop stack frame
+                self.stack.pop();
+            }
+        }
+    }
+}
+
+impl<'a, T: Component> Iterator for DepthFirstIterator<'a, T> {
     type Item = Entity;
 
     fn next(&mut self) -> Option<Self::Item> {
@@ -138,16 +231,16 @@ impl<'a, W: GenericWorld, T: 'static + Send + Sync> Iterator for DepthFirstItera
         if top.remaining > 0 {
             let current = top.current;
 
-            let data = self.world.try_get::<Child<T>>(top.current).ok().unwrap();
+            let data = self.children.get(top.current).ok().unwrap();
 
             // Go to the next child in the linked list of children
             top.current = data.next;
             top.remaining -= 1;
 
             // If current is a parent, push a new stack frame with the first child
-            if let Ok(parent) = self.world.try_get::<Parent<T>>(current) {
+            if let Ok(parent) = self.parents.get(current) {
                 self.stack.push(StackFrame {
-                    current: parent.first_child(self.world).unwrap(),
+                    current: parent.column_first_child(&self.children).unwrap(),
                     remaining: parent.num_children,
                 })
             }
